import pytest
import numpy as np

import dace
from dace import nodes
from dace.transformation import interstate
from dace.sdfg import utils as sdfg_utils
import dace.library
from dace.libraries import blas

from daceml.util import utils
from daceml.distributed import schedule
from daceml.distributed.utils import find_map_containing, compile_and_call, arange_with_size

blas.default_implementation = 'OpenBLAS'

size = 'half2'
sizes = [8, 4, 4]

N = 4096 * 2
K = 4096 * 2
M = 4096 * 2

@dace
def matmul(a: dace.float32[N, K], b: dace.float32[K, M]):
    return a @ b

sdfg = matmul.to_sdfg()
sdfg.name = sdfg.name + str(size)
utils.expand_nodes(sdfg, predicate=lambda n: isinstance(n, blas.MatMul))
cands = [
    n for n in sdfg.node(0).nodes()
    if isinstance(n, nodes.LibraryNode)
]
assert len(cands) == 1
lib = cands[0]
scheme = {lib: {"gemm_map": sizes, "init_map": [sizes[0], sizes[-1]]}}
schedule.lower(sdfg, scheme)
#sdfg.apply_gpu_transformations()

A = np.random.rand(N, K).astype(np.float32)
B = np.random.rand(K, M).astype(np.float32)

expected = A @ B
compile_and_call(sdfg, {
    'a': A.copy(),
    'b': B.copy()
}, expected, utils.prod(sizes))
print(f"Passed using schedule {scheme}!")
